// ****************************************************
// User Defined File Loader Standard Structure
//
// Proposed Standard
// J. J. Weimer
// History: $Revision: 803 $ on $Date: 2008-07-07 11:07:57 -0500 (Mon, 07 Jul 2008) $
//
// in addition to using this structure
// "standard" file loaders should
//		return 0 if successful
//		return -1 if unsuccessful

#pragma rtGlobals = 1
#pragma IgorVersion=6.03
#pragma version = 2.0

STRUCTURE udFLStandardStructure

	// descriptors
	//------------------
	// ... these define and are to be returned
	// ... by the loader when userCtrl = 0
	
	string mimetype			// for example "TEXT"
	string extensions			// for example ".txt;.dat;.fil;"
	int32 itemsType			// bit value type of loaded item that is returned (see below)
	
		// The value of itemsType may depend on userCtrl
		// In such cases, when userCtrl = 0, the bitwise additive
		// value of itemsDim should be returned. For userCtrl != 0,
		// when file loading has completed, itemsDim must be set
		// to the proper bitwise additive value of return types
		// prior to return from the loader.
		// ...
		// BIT
		// 	0: keyword-list (nominally using "=" and ";") (ie, itemsType = 1)
		// 	1: waves (ie, itemsType = 2)
		//	2: matrix (ie, itemsType = 4)
		//	3: image (ie, itemsType = 8)
		//	4: notebook (ie, itemsType = 16)
		//	5: procedure file (ie, itemsType = 64)
		//
		// EXAMPLE
		//	itemsType = 3 means keyword-list and/or waves are returned
		//	itemsType = 80 means notebook and procedure file are returned

	int32 itemsDim[4]			// "dimension" of loaded item
	
		// This value adds additional information to the itemsType
		// Always try to return a proper value of itemsDim with userCtrl = 0
		// and itemsType = (given value) so that a user query for the
		// proper post-processing of the given loaded item in each case
		// When a certain type is not loaded, return itemsDim[N] = 0
		// When the "dimension" of an item depends on other parts of the
		// process, return itemsDim[N] = 99 (undefined)
		// itemsType			itemsDim
		//	1							[0]:	1: uses "="	2: uses ":"
		//	2					(negative itemsDim indicate TEXT waves)
		//								[0]: 	+/- 1: unscaled
		//								[0]: 	+/- 2: scaled
		//	4			(negative itemsDim indicate TEXT in given dimension)
		//						use the same convention for waves
		//						use [index] for dimension of matrix
		//					[0]: row	[1]: column	[2]: layer	[3]: chunk
		//	8							>> to be defined <<
		//	16							[0]: 	1: unformated		2: formatted
		//	64							>> to be defined <<
		
	string loadModes		// list of loading modes that this loader provides
	
		// This string list of the loading modes
		// They are defined by userLCtrl starting as
		// ...
		// -1: Initialize
		// 0: Query ONLY
		// 1: "Auto" - all needed structure parameters are pre-defined
		// others: defined by programmer

	string procModes		// list of processing modes that this loader provides
	
		// This string list of the processing modes
		// They are defined by userPCtrl, starting as
		// ...
		// -1: Initialize
		// 0: Query ONLY
		// others: defined by file loader
		
	int32 setNames			// list of ways this loader can rename inputs
	
		// This is a BIT-wise variable that defines the possible ways
		// the user can (re)name what is loaded. When any one way
		// is supported at any procMode, it should be supported
		// by all procModes. This value is both a Descriptor and an Input
		//
		// BIT
		//	0:	loader sets input names ... user has no possible control
		//	1:	user can (re)name input according to file names
		//	2:	user can have loader (re)name inputs according to a file header
		//	3:	user can (re)name input according to prefix + autonumeric increment
		//	4:	user can manually (re)name each input (via dialog)
		//	5:	loader can define its own way (see below)
		//
		// when all setName options are supported, the value of
		// udFL.setNames will be (1+2+4+8+16+32) = 63
		
		// For a loader to define its own method of setting a name, use
		// udFLsetNames = 32 and udFL.userData = METHOD where METHOD is a (short) string

	// inputs
	//--------------
	
	int32	eventCode			// what "event" is calling the loader

			//  -1: request to kill the loader function
			// others: user defined
			
	int32	userLCtrl				// load control variable
	
			// This value is always an integer input by user
			// It defines the loadModes in the following way:
			// -1: initialize loader (do this FIRST!)
			// 0: query loader
			// 1: load via "auto" load methods
			// others: user defined processing options

	int32	userPCtrl				// process control variable
	
			// This value is always an integer input by user
			
	string userData			// user string input/loader output string
	
			// use userData (string) to provide additional manual "tweaks" to loader as needed
			
	int32 reportCtrl		// control on progress reporting
			
			// -1: silent mode
			// 0: normal mode
			// 1: verbose mode
			// others: user defined "reporting" modes
			
	string plugIns				// string list of "plug-in" functions to try for processing
		
	string namePrefix			// the prefix string for setName = 8
	
	// localizers
	// -----------------
	
	string pathStr				// path to folder
	string fileList				// list of files, for example "test.txt;scan.txt;"
	int32 returnCtrl			// control return values
	
			// 0: normal mode (no extra variables required)
			// 1: also create S_filename, V_flag, and S_value
			// others: user defined return modes
			
	string toFolder			// full path to folder where data are to be loaded
	
	// returns
	// ---------
	
	int32 itemsCount				// number of items loaded
	STRUCT itemsList itLst		// structured list of items loaded
	int32 errCode					// error code, if any
	
			// 0: warning message only
			// -1: potentially fatal abort error
			
	string errMsg					// error message string

ENDSTRUCTURE

STRUCTURE itemsList
	string GLOBALS
	string LISTS
	string WAVES
	string MATRICES
	string IMAGES
	string NOTEBOOKS
	string PROCEDURES
ENDSTRUCTURE